/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.sensor.greenseeker;

import com.linuxense.javadbf.DBFException;
import com.linuxense.javadbf.DBFReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Logger;
import nl.cloudfarming.client.sensor.SensorDataObject;
import nl.cloudfarming.client.sensor.SensorProcessor;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiFileLoader;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle.Messages;

/**
 * GreenSeeker DataObject. Tries to detect v/d Borne and Normal file processors
 * based on the number of fields in the sensor file.
 *
 * @author Arjan Loermans
 */
@Messages({
    "# Fileprocessor", "sensor.greenseeker.fileprocessor.borne.log.displayname=Greenseeker Borne", "sensor.greenseeker.fileprocessor.normal.log.displayname=Greenseeker Normal", "# Processorprovider", "sensor.greenseeker.processorprovider.filefilter.description=Greenseeker data {0}", "# Logerror", "sensor.greenseeker.fileprocessor.logerror.failed_to_create_time=Could not convert {0} to time."
})
public class GreenseekerDataObject extends SensorDataObject {

    private GreenseekerFileProcessor processor = null;
    int gsType = 0;
    private static final Logger LOG = Logger.getLogger("nl.cloudfarming.client.sensor.greenseeker.GreenseekerDataObject");

    public GreenseekerDataObject(FileObject pf, MultiFileLoader loader) throws DataObjectExistsException, IOException {
        super(pf, loader);
        LOG.info("GreenseekerDataObject constructor");
        registerEditor("application/dbf-greenseeker", true);
        this.gsType = fieldCount(pf);
        LOG.info("GreenseekerDataObject constructor done");
    }

    @Override
    protected int associateLookup() {
        return 1;
    }

    @Override
    public HelpCtx getHelpCtx() {
        return new HelpCtx("nl.cloudfarming.client.sensor.greenseeker.about");
    }

    private int fieldCount(FileObject fo) throws FileNotFoundException, DBFException, IOException {
        InputStream inputStream = new FileInputStream(FileUtil.toFile(fo));
        DBFReader reader = new DBFReader(inputStream);
        int retVal = reader.getFieldCount();
        inputStream.close();
        return retVal;
    }

//    @MultiViewElement.Registration(displayName = "#LBL_Greenseeker_EDITOR",
//    iconBase = "nl/cloudfarming/client/sensor/greenseeker/file_extension_qxd.png",
//    mimeType = "application/dbf-greenseeker",
//    persistenceType = TopComponent.PERSISTENCE_ONLY_OPENED,
//    preferredID = "Greenseeker",
//    position = 1000)
//    @Messages("LBL_Greenseeker_EDITOR=Editor")
//    public static MultiViewEditorElement createEditor(Lookup lkp) {
//        return new MultiViewEditorElement(lkp);
//    }

    @Override
    protected SensorProcessor getProcessor() {
        if (this.processor == null) {
            // Check the number of fields in the file, Borne has 20 fields, Normal has 14. Set the corresponding Processor
            switch (this.gsType) {
                case GreenseekerFileProcessorBorne.nrOfFields:
                    processor = new GreenseekerFileProcessorBorne(this);
                    break;
                case GreenseekerFileProcessorNormal.nrOfFields:
                    processor = new GreenseekerFileProcessorNormal(this);
                    break;
                default:
                    processor = null;
                    break;
            }
        }
        return this.processor;
    }
}
